;+ 
; NAME:
; mike_allflat   
;     Version 1.1
;
; PURPOSE:
;    Identifies flats, processes them.  Creates one flat file
;      per slit width.  Takes TFLAT as the default
;
; CALLING SEQUENCE:
;   
;  mike_mktflat, mike
;
; INPUTS:
;   mike     -  ESI structure
;   [slit]  -  Slit size (e.g. 0.5, 0.75, 1.0)
;
; RETURNS:
;
; OUTPUTS:
;  One normalized flat per slit width
;
; OPTIONAL KEYWORDS:
;   /REDOOV  - Overwrite OV files if they exist for the flats
;   /SVOV    - Save the OV files created during this step
;   /TFCHK   - Perform careful checking of the trace flat
;   /CHK     - Check the 2D residuals
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   mike_allflat, mike, 1.0
;
; PROCEDURES/FUNCTIONS CALLED:
;
; REVISION HISTORY:
;   14-Aug-2003 Written by JXP
;-
;------------------------------------------------------------------------------


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro mike_allflat, mike, setup, side, REDOOV=redoov,SVOV=svov,CLOBBER=clobber, $
                  CHK=chk, INTER=inter, TFCHK=tfchk, _EXTRA=extra

;
 if  N_params() LT 2  then begin 
      print,'Syntax - ' + $
        'mike_allflat, mike, setup, [side], /REDOOV, /clobber [v1.0]'
      return
  endif 
  
;  Optional Keywords
  if not keyword_set( SIDE ) then side = [1L,2L]
  if not keyword_set( clobber ) then clobber = 0L
  if not keyword_set( iflat ) then iflat = 0L
 

  for i=0, n_elements(side) -1 do begin

    qq = side[i]
 
    ; Milky Flat
    mike_mkmflat, mike, setup, qq, CLOBBER=clobber

   ; Trace flat
    ;; COMBINE
     mike_mktflat, mike, setup, qq, CLOBBER=clobber

    ;; TRACE
    mike_trcflat, mike, setup, qq, CLOBBER=clobber, INTER=inter, CHK=tfchk, $
      _EXTRA=EXTRA

    ;; FIT TRACE
    mike_fittflat, mike, setup, qq, CLOBBER=clobber

  endfor

  ;; SLIT PROFILE  
  ;;  Call mike_sliflat manually after mike_allarc
  print, 'Call mike_slitflat manually after mike_allarc:: '
  print, 'IDL> mike_slitflat, mike, setup, CLOBBER=clobber'

  print, 'mike_allflat: All done!'
  return
end
