
pro mtrace, image, tset, tracefit, edge=edge, meas=meas, traceimage=traceimage, $
     trace=trace, ytrace=ytrace, ncoeff=ncoeff

    ncol = (size(image))[1]
    nrow = (size(image))[2]

    if NOT keyword_set(ncoeff) then ncoeff=5L

;
;  Filter image with vertical median
;
    imedvertical = image * 0.0
    width = long(42.0 * nrow / 4096) 
    print, 'Median filter is ', width
    col20 = fltarr(ncol) 
    for i=0,ncol -1 do begin
       imedvertical[i,*]=djs_median((image[i,*])[*],width=width)
       col20[i] = imedvertical[i,(sort(imedvertical[i,*]))[nrow/5]]
    endfor
       

    gsigma = 3.0 * ncol / 2048 
    radius = 3 * gsigma
    hpix = long(radius)
     
    x = findgen(ncol-100) + 50
    
    image20 = col20 # replicate(1,nrow) / 2.0
    iback = total(imedvertical-image20, 2) / nrow
    hmm = bspline_iterfit(findgen(ncol), iback, nbkpt=5, yfit=backfit)

    mn = mean(backfit)
    backfit = (backfit > (mn/10.0)) / mn
    backimage = backfit # replicate(1,nrow)

    sawtooth = [-1., -1., 0., 1., 1.]

    if keyword_set(edge) then begin
      edge = convol(imedvertical - image20, sawtooth)
      traceimage = edge/backimage
    endif else begin 
;
;   let's try a gaussian filter horizontally
;
      print, 'Gaussian filter with ', gsigma*2.345, ' FWHM'
      gkernel = gauss_kernel(gsigma, hpix=hpix)

      ismooth = convol([fltarr(hpix,nrow),imedvertical, fltarr(hpix,nrow)], gkernel)
      ismooth = ismooth[hpix:ncol+hpix-1,*] - image20
      traceimage = ismooth/backimage
      traceraw = image/backimage
    endelse

    thresh = traceimage[(sort(traceimage[*,nrow/2]))[ncol*0.7], nrow/2]


    trace = trace_crude(traceimage, yset=ytrace, thresh=thresh, xerr=traceerr, $
             maxshifte=0.5,radius=radius, nave=width)

    traceerr[0:width/2,*] = 999
    traceerr[nrow-width/2:*,*] = 999

    xy2traceset, ytrace, trace, tset, $
         invvar=(traceerr LT 900), maxrej=20, maxiter=40, $
         maxdev=5.0, outmask=tracemask, yfit=interfit, ncoeff=3

    meas = trace_fweight(traceimage, interfit, ytrace, xerr=measerr,radius=radius)
    meas = trace_fweight(traceimage,  meas, ytrace, xerr=measerr,radius=radius)
    measerr[0:width/2,*] = 999
    measerr[nrow-width/2:*,*] = 999


    xy2traceset, ytrace, double(meas), tset, $
         invvar=(measerr LT 900) * $
             (interfit GT 0) * (interfit LT ncol - 1) ,  $
         outmask=tracemask, yfit=tracefit, ncoeff=ncoeff

    flux = extract_boxcar(imedvertical, tracefit)

    keeptrace = where(total(tracemask AND (tracefit GT 2) AND $
                           (tracefit LT ncol-3),1) GT nrow/2., ntrace)

    if ntrace EQ 0 then return


    xy2traceset, ytrace[*,keeptrace], double(meas[*,keeptrace]), tset, $
         invvar=tracemask[*,keeptrace], yfit=tracefit, ncoeff=ncoeff
    return
end

