pro xset, file1, file2, outfile=outfile, doplot=doplot

     pl1 = rstrpos(file1,'.fits')-5
     if pl1 LT 1 then begin
       print, 'Cannot identify fits file #1: ', file1
       return
     endif

     color1 = strmid(file1,pl1,1)
     if (color1 NE 'r' AND color1 NE 'b') then begin
        print, 'Cannot identify the color: ', color1
        return
     endif

     dual = 0L
     color2 = ''
     if (size(file2,/tname)) EQ 'STRING' then begin
        pl2 = rstrpos(file2,'.fits')-5
        if pl2 GE 1 then begin
          color2 = strmid(file2,pl2,1)
          if color2 NE color1 then begin
            print, 'Colors do not match: ', color1, ' vs. ', color2
            return
          endif        
          dual = 1L
        endif
     endif 

     if NOT keyword_set(outfile) then outfile = 'mike_'+ $
          (color1 EQ 'r' ? 'red' : 'blue') + '_xset.tab'

     mproc, file1, image1, hdr=hdr1
     ncol = (size(image1))[1]
     nrow = (size(image1))[2]
     print, 'Tracing ', file1, ' from scatch...'
     mtrace, image1, tset1, tfit1, ytrace=yfit1, ncoeff=ncoeff
     ntrace1 = (size(tfit1))[2]
     print, ' Found ',  ntrace1, ' good orders' 
     if keyword_set(doplot) then plot, tfit1, yfit1, ps=3

     distortion = [1.0, 0.0]
     tset1.coeff = tset1.coeff/ncol 


     if dual EQ 0 then begin
        print, 'Overwriting xset file: ', outfile
        print, '  No distortion information, since only one file is traced'
     endif else begin

     mproc, file2, image2, hdr=hdr2
     ncol2 = (size(image2))[1]
     nrow2 = (size(image2))[2]
     print, 'Tracing ', file2, ' from scatch...'
     mtrace, image2, tset2, tfit2, ytrace=yfit2, ncoeff=ncoeff
     ntrace2 = (size(tfit2))[2]
     print, ' Found ',  ntrace2, ' good orders' 
     if keyword_set(doplot) then plot, tfit2, yfit2, ps=3

     tset2.coeff = tset2.coeff/ncol2


     if tset2.coeff[0,0] LT tset1.coeff[0,0] then begin
       tempset = tset1
       tset1 = tset2
       tset2 = tempset
     endif

     ntrace1 = (size(tset1.coeff))[2]
     ntrace2 = (size(tset2.coeff))[2]
 
;
;   let's match traces
;
     for i=0,ntrace1-1 do begin
        diff = (tset2.coeff[0,*]-tset1.coeff[0,i])
        check = where(diff LT 0.02 AND diff GT 0.005)
       if check[0] NE -1 then begin
          if N_elements(m1) EQ 0 then begin
            m2 = check[0]
            m1 = i
          endif else begin
            m2 = [m2, check[0]]
            m1 = [m1, i]
          endelse
       endif
     endfor
 
     

     slitw = tset2.coeff[0,m2] - tset1.coeff[0,m1]
     slitnorm = slitw/median(slitw)

     distortion = ladfit(tset1.coeff[0,m1], slitnorm)
     slitfit = poly(tset1.coeff[0,m1],distortion)

     matches = where(abs(slitfit-slitnorm) LT 0.02, nmatches)
     if matches[0] EQ -1 then begin
       print, 'Could not match traces between two exposures'
       return
     endif

     print, 'Matched ', nmatches, ' Orders between the two frames'

     aveset = { func: tset1.func, xmin : tset1.xmin , xmax : tset1. xmax, $
                coeff : fltarr(ncoeff,nmatches) }
     
     aveset.coeff = (tset1.coeff[*,m1[matches]] + tset2.coeff[*,m2[matches]])/2.0
  

     print, 'Overwriting xset file: ', outfile
     print, '  Distortion polynomial was found ', distortion[*], format='(a,3(f8.4))'

     endelse

     openw, ilun, outfile, /get_lun
     printf, ilun, '#Distortion correction'
     printf, ilun, 2
     printf, ilun, distortion[*]
     printf, ilun, '#trace polynomials, legendre terms in fraction of CCD'
     printf, ilun, ncoeff, nmatches
     printf, ilun, aveset.coeff
     free_lun, ilun

;     mwrfits, distortion, outfile, /create
;     mwrfits, aveset, outfile
     
     return
end
