pro xyproc, filename, outfile, xsetfile=xsetfile, ysetfile=ysetfile, $
             slitfinal=slitfinal 

    if NOT keyword_set(slitfinal) then slitfinal=0L

    if NOT keyword_set(outfile) then $
       outfile = strmid(filename,0,rstrpos(filename,'.fits'))+'.rect.fits'

    if NOT keyword_set(xsetfile) OR NOT keyword_set(ysetfile) then begin
      pl1 = rstrpos(filename,'.fits')-5
      if pl1 LT 1 then begin
         print, 'Cannot identify fits : ', filename
         return
      endif

      color1 = strmid(filename,pl1,1)
      if (color1 NE 'r' AND color1 NE 'b') then begin
          print, 'Cannot identify the color: ', color1
          return
      endif

      if NOT keyword_set(xsetfile) then xsetfile = 'mike_'+ $
          (color1 EQ 'r' ? 'red' : 'blue') + '_xset.tab'
      if NOT keyword_set(ysetfile) then ysetfile = 'mike_'+ $
          (color1 EQ 'r' ? 'red' : 'blue') + '_yset.tab'
    endif

;
;   Bias and overscan subtract file and store image size in ocol, orow
;
    mproc, filename, image, hdr=hdr
    ocol = (size(image))[1]
    orow = (size(image))[2]
;
;   If desired, operate on image to remove cosmic rays here
;

;
;
    xrec = xproc(image, xsetfile, slitfinal=slitfinal, hdr=hdr)

    ncol = (size(xrec))[1]
    nrow = (size(xrec))[2]
    norder = nrow / (slitfinal+2)
;
;   now do y-correction
;
    commentline = strarr(1)
    openr, ilun, ysetfile, /get_lun
    readf, ilun, commentline
    readf, ilun, nxcoeff, nycoeff
    coeff_fit = fltarr(nxcoeff, nycoeff)
    readf, ilun, coeff_fit
    free_lun, ilun 

    coeff_fit  = coeff_fit * (ocol * 2/ ncol)
    if n_elements(coeff_fit) LE 1 then begin
      print, 'No tilt coefficients found in ', ysetfile
      return
    endif

    iorder = lindgen(norder)
    tilt_coeff = fltarr(2,norder)
    tilt_coeff[0,*] = poly(iorder, coeff_fit[*,0])  
    tilt_coeff[1,*] = poly(iorder, coeff_fit[*,1]) 

    finalrec = xrec
    os = iorder*(slitfinal+2)
    x = findgen(ncol)
    y = findgen(slitfinal) - (slitfinal/2)

    for i=0,norder-1 do begin
       tiltarray = poly(x/ncol, tilt_coeff[*,i])
       xtrace = tiltarray ## y + x ## replicate(1,slitfinal)
       temprec = extract_boxcar(xrec[*,os[i]+2:os[i]+slitfinal+1],xtrace,radius=0.5)
       finalrec[*,os[i]+2:os[i]+slitfinal+1] = transpose(temprec)
    endfor
   
    sxaddpar, hdr, 'RECTIFY', 1L, 'Frame has been rectified'
    sxdelpar, hdr, 'BSCALE'
    sxdelpar, hdr, 'BZERO'
  
    mwrfits, finalrec, outfile, hdr, /create

    return
end
