pro yset, arcfile, medbin=medbin, xsetfile=xsetfile, outfile=outfile, $
          slitfinal=slitfinal

     if NOT keyword_set(slitfinal) then slitfinal=0L

     pl1 = rstrpos(arcfile,'.fits')-5
     if pl1 LT 1 then begin
       print, 'Cannot identify fits file #1: ', file1
       return
     endif

     color1 = strmid(arcfile,pl1,1)
     if NOT keyword_set(xsetfile) then xsetfile = 'mike_'+ $
          (color1 EQ 'r' ? 'red' : 'blue') + '_xset.tab'

     if NOT keyword_set(outfile) then outfile = 'mike_'+ $
          (color1 EQ 'r' ? 'red' : 'blue') + '_yset.tab'

;
;   Bias and overscan subtract file and store image size in ocol, orow
;
     mproc, arcfile, image, hdr=hdr
     ocol = (size(image))[1]
     orow = (size(image))[2]

     recarc = xproc(image, xsetfile, slitfinal=slitfinal)
     osep = slitfinal

     
     if NOT keyword_set(medbin) then medbin=3L

     nrow = (size(recarc))[2]
     ncol = (size(recarc))[1]

     
     colsum = total(recarc,1)/nrow
     norder = nrow/(osep+2)
     orderstart = lindgen(norder)*(osep+2)


     if norder LT 2 then begin
       print, 'I could not find any orders'
       return
     endif

     orderfullsep = 2+osep
     top_spot     = fltarr(norder)
     tilt0        = fltarr(norder)
     tilt_coeff   = fltarr(2,norder)
   
     for i=0,norder-1 do begin
 
       orderimg = recarc[*,orderstart[i]+2:orderstart[i]+osep+1]
       medcols = long(ncol/medbin)*medbin
      
       colsquash = djs_median(reform(orderimg[0:medcols-1,*], $
                                     medbin,ncol/medbin,osep),1)
       colsum = total(colsquash,2)
       colsort = sort(colsum)
       usecols = where(colsum GT 1.5*median(colsum) $
                 AND total(colsquash GT 60000,2) EQ 0 ,nuse)
 
       if nuse LT 10 then begin
          print, 'Could not find profile in order ', i
          return
       endif

       profile = total(colsquash[usecols,*],1) / $
                 (total(colsquash[usecols,*] GT 0,1) + 1) 
       plot,profile

       peak = (osep/4+1) - abs(findgen(osep/2+1) - (osep/4+1))
       bottom_max = max(profile[0:osep/2]*peak,bottoms)
       bottom_spot = bottoms 
       bottom = djs_median(orderimg[*,bottom_spot-1:bottom_spot+1],2)

       top_max = max(profile[osep/2:*]*reverse(peak),tops)
       top_spot[i] = tops + osep/2 
       top    = djs_median(orderimg[*,top_spot[i]-1:top_spot[i]+1],2)

       delta_rows = top_spot[i] - bottom_spot
       lags = findgen(41)-20.
       cc = c_correlate(top,bottom,lags)
       gausscc = gaussfit(lags,cc,gauss_coeff,nterms=5)
       delta_cols = gauss_coeff[1]

       if (delta_cols LT -10 OR delta_cols GE 10) then begin
           print, 'Poorly defined initial tilt in order ', i 
           return
       endif
      
       tilt0[i] = -1.0 * (delta_cols/delta_rows) 
       trace = trace_crude(orderimg, yset=ytrace, thresh=2.0*median(orderimg), $
                      ystart=top_spot[i], maxshifte=0.3,radius=3.)

       nstart = (size(trace))[2]
       xstart = tilt0[i]*(findgen(osep) - long(top_spot[i]))#replicate(1,nstart) + $
                 replicate(1,osep)#trace[top_spot[i],*]

       trace2 = trace_fweight(orderimg, xstart, ytrace,radius=3.)
       trace3 = trace_fweight(orderimg, trace2, ytrace,radius=3.)
       trace4 = trace_fweight(orderimg, trace3, ytrace,radius=3.)
       err = (trace4 - xstart)
       weight = 1.0/(err > 0.05)^2 * (err LT 0.5)
       xy2traceset, ytrace, trace4, tiltset, invvar=weight, ncoeff=2, $
         upper=5, lower=5, yfit=tiltfit,maxrej=1,maxiter=20
      
       tilt_coeff[*,i] = ladfit(tiltset.coeff[0,*]/ncol, $
                   tiltset.coeff[1,*]/(tiltset.xmax/2)) 
       
       if NOT keyword_set(xkeep) then begin
          ykeep = replicate(i,norder)
          xkeep = (tiltfit[osep/2,*]/ncol)[*]
       endif else begin
          ykeep = [ykeep,replicate(i,norder)]
          xkeep = [xkeep, (tiltfit[osep/2,*]/ncol)[*]]
       endelse 
    endfor

    inside = lindgen(norder-4) + 1
    coeff_fit= fltarr(2,2)
    coeff_fit[*,0] = ladfit(inside, tilt_coeff[0,inside])
    coeff_fit[*,1] = ladfit(inside, tilt_coeff[1,inside])

    coeff_fit = coeff_fit * (ncol/2.0 /ocol)

    print, 'Success! writing arc/tilt coefficients to ', outfile

    openw, ilun, outfile, /get_lun
    printf, ilun, '#yset distortion fit coefficients'
    printf, ilun, 2, 2
    printf, ilun, coeff_fit
    free_lun, ilun

  
;    mwrfits, coeff_fit, outfile, /create

    return
end

